@@ -1,6 +1,7 @@
 // ! CircuitNodeInit and various helpers for nicely implementing it
 use std::{collections::BTreeMap, iter::zip, sync::Arc};
 
+use anyhow::Context;
 use rr_util::{
     name::Name,
     symbolic_size::{SymbolicSizeConstraint, SymbolicSizeConstraints},
@@ -63,8 +64,17 @@ pub trait CircuitNodeComputeInfoImpl: CircuitNode {
         self.children()
             .map(|c| c.info().device_dtype.clone())
             .chain(self.device_dtype_extra())
-            .fold(Ok(TorchDeviceDtypeOp::NONE), |acc, new| {
-                acc.map(|old| TorchDeviceDtypeOp::combine(old, new))?
+            .try_fold(TorchDeviceDtypeOp::NONE, |acc, new| {
+                TorchDeviceDtypeOp::combine(acc, new)
+            })
+            .with_context(|| {
+                format!(
+                    "Could not combine device_dtype for node {:?} with children {:?}",
+                    self.info(),
+                    self.children()
+                        .map(|c| c.info().clone())
+                        .collect::<Vec<_>>()
+                )
             })
     }
 
