@@ -7,6 +7,7 @@ use pyo3::{
 };
 use rr_util::{
     lru_cache::TensorCacheRrfs,
+    name::Name,
     py_types::{scalar_to_tensor, Tensor, PY_UTILS},
     sv,
     symbolic_size::{shape_first_symbolic_dim, SymbolicSizeProduct},
@@ -51,7 +52,6 @@ pub struct Array {
     #[pyo3(get)]
     pub value: Tensor,
     info: CachedCircuitInfo,
-    name: Option<String>,
 }
 
 circuit_node_extra_impl!(Array, self_hash_default);
@@ -87,7 +87,7 @@ impl CircuitNode for Array {
 
 impl Array {
     #[apply(new_rc_unwrap)]
-    pub fn try_new(value: Tensor, name: Option<String>) -> Result<Self> {
+    pub fn try_new(value: Tensor, name: Option<Name>) -> Result<Self> {
         let value = value.hashed();
         if let Some(i) = shape_first_symbolic_dim(value.shape()) {
             bail!(ConstructError::ArrayHasReservedSymbolicShape {
@@ -95,15 +95,15 @@ impl Array {
                 sym: SymbolicSizeProduct::from(value.shape()[i]).to_string(),
             })
         }
-        Self {
+        let mut out = Self {
             value,
-            name,
             info: Default::default(),
-        }
-        .initial_init_info()
+        };
+        out.info.name = name;
+        out.initial_init_info()
     }
     pub fn from_hash_prefix(
-        name: Option<String>,
+        name: Option<Name>,
         hash_base16: &str,
         tensor_cache: &mut Option<TensorCacheRrfs>,
     ) -> Result<Self> {
@@ -119,11 +119,7 @@ impl Array {
         Self::randn_full(shape, None, Default::default(), None)
     }
 
-    pub fn randn_named(
-        shape: Shape,
-        name: Option<String>,
-        device_dtype: TorchDeviceDtypeOp,
-    ) -> Self {
+    pub fn randn_named(shape: Shape, name: Option<Name>, device_dtype: TorchDeviceDtypeOp) -> Self {
         Self::randn_full(shape, name, device_dtype, None)
     }
 }
@@ -132,7 +128,7 @@ impl Array {
 impl Array {
     #[new]
     #[args(name = "None")]
-    fn py_new(value: Tensor, name: Option<String>) -> Result<PyClassInitializer<Self>> {
+    fn py_new(value: Tensor, name: Option<Name>) -> Result<PyClassInitializer<Self>> {
         Ok(Array::try_new(value, name)?.into_init())
     }
 
@@ -146,7 +142,7 @@ impl Array {
     )]
     pub fn randn_full(
         shape: Shape,
-        name: Option<String>,
+        name: Option<Name>,
         device_dtype: TorchDeviceDtypeOp,
         seed: Option<usize>,
     ) -> Self {
@@ -195,14 +191,14 @@ impl Array {
     }
 
     #[staticmethod]
-    pub fn from_hash(name: Option<String>, hash_base16: &str) -> Result<Self> {
+    pub fn from_hash(name: Option<Name>, hash_base16: &str) -> Result<Self> {
         get_tensor_prefix(hash_base16).map(|value| Array::new(value, name))
     }
 
     #[staticmethod]
     #[pyo3(name = "from_hash_prefix")]
     pub fn from_hash_prefix_py(
-        name: Option<String>,
+        name: Option<Name>,
         hash_base16: &str,
         tensor_cache: Option<TensorCacheRrfs>,
     ) -> Result<Self> {
@@ -217,7 +213,6 @@ pub struct Symbol {
     #[pyo3(get)]
     pub uuid: Uuid,
     info: CachedCircuitInfo,
-    name: Option<String>,
 }
 
 circuit_node_extra_impl!(Symbol, self_hash_default);
@@ -257,13 +252,13 @@ impl CircuitNode for Symbol {
 
 impl Symbol {
     #[apply(new_rc)]
-    pub fn new(shape: Shape, uuid: Uuid, name: Option<String>) -> (Self) {
+    pub fn new(shape: Shape, uuid: Uuid, name: Option<Name>) -> (Self) {
         let mut out = Self {
             uuid,
-            name,
             info: Default::default(),
         };
         out.info.shape = shape;
+        out.info.name = name;
         out.initial_init_info().unwrap()
     }
 }
@@ -271,23 +266,23 @@ impl Symbol {
 #[pymethods]
 impl Symbol {
     #[new]
-    fn py_new(shape: Shape, uuid: Uuid, name: Option<String>) -> PyClassInitializer<Self> {
+    fn py_new(shape: Shape, uuid: Uuid, name: Option<Name>) -> PyClassInitializer<Self> {
         Symbol::new(shape, uuid, name).into_init()
     }
 
     #[staticmethod]
-    pub fn new_with_random_uuid(shape: Shape, name: Option<String>) -> Self {
+    pub fn new_with_random_uuid(shape: Shape, name: Option<Name>) -> Self {
         Self::new(shape, Uuid::new_v4(), name)
     }
     #[staticmethod]
-    pub fn new_with_none_uuid(shape: Shape, name: Option<String>) -> Self {
+    pub fn new_with_none_uuid(shape: Shape, name: Option<Name>) -> Self {
         Self::new(shape, Uuid::nil(), name)
     }
 
     /// equality by ident is nice because we can guarantees about correct
     /// behavior even when shapes change
-    pub fn ident(&self) -> (Uuid, Option<&str>) {
-        (self.uuid, self.name())
+    pub fn ident(&self) -> (Uuid, Option<Name>) {
+        (self.uuid, self.info().name)
     }
 }
 
@@ -297,7 +292,6 @@ pub struct Scalar {
     #[pyo3(get)]
     pub value: f64,
     info: CachedCircuitInfo,
-    name: Option<String>,
 }
 
 circuit_node_extra_impl!(Scalar, self_hash_default);
@@ -331,12 +325,12 @@ impl CircuitNode for Scalar {
 
 impl Scalar {
     #[apply(new_rc)]
-    pub fn new(value: f64, shape: Shape, name: Option<String>) -> (Self) {
+    pub fn new(value: f64, shape: Shape, name: Option<Name>) -> (Self) {
         let mut out = Self {
             value,
-            name,
             info: Default::default(),
         };
+        out.info.name = name;
         out.info.shape = shape;
 
         out.initial_init_info().unwrap()
@@ -347,7 +341,7 @@ impl Scalar {
 impl Scalar {
     #[new]
     #[args(shape = "sv![]")]
-    fn py_new(value: f64, shape: Shape, name: Option<String>) -> PyClassInitializer<Self> {
+    fn py_new(value: f64, shape: Shape, name: Option<Name>) -> PyClassInitializer<Self> {
         Self::new(value, shape, name).into_init()
     }
 
@@ -360,7 +354,7 @@ impl Scalar {
     }
 
     pub fn evolve_shape(&self, shape: Shape) -> Self {
-        Self::new(self.value, shape, self.name.clone())
+        Self::new(self.value, shape, self.info().name)
     }
 }
 
