@@ -16,7 +16,8 @@ use rr_util::{
     py_types::{assert_tensors_close, ExtraPySelfOps, PyShape, Tensor, PY_UTILS},
     pycall, python_error_exception, simple_from,
     tensor_util::{
-        check_canon_idxs, MiscInputError, Shape, TorchDeviceDtype, TorchDeviceDtypeOp, TorchDtype,
+        broadcast_shapes, check_canon_idxs, MiscInputError, Shape, TorchDeviceDtype,
+        TorchDeviceDtypeOp, TorchDtype,
     },
 };
 use rustc_hash::{FxHashMap as HashMap, FxHashSet as HashSet};
@@ -62,7 +63,10 @@ macro_rules! gf_gen {
     };
 }
 
+// tuples correspond to GeneralFunctionSimpleSpec fields (default removed_from_end = 0)
 gf_gen!(
+    (sin, 0),
+    (cos, 0),
     (sigmoid, 0),
     (tanh, 0),
     (rsqrt, 0),
@@ -500,6 +504,134 @@ impl SpecTrait for GeneralFunctionSetDDSpec {
     }
 }
 
+#[pyclass]
+#[derive(Debug, Clone)]
+pub struct GeneralFunctionPowSpec {}
+
+simple_from!(|x: GeneralFunctionPowSpec| -> GeneralFunctionSpec { GeneralFunctionSpec::Pow(x) });
+
+impl ToPyObject for GeneralFunctionPowSpec {
+    fn to_object(&self, py: Python<'_>) -> PyObject {
+        self.clone().into_py(py)
+    }
+}
+
+impl SpecTrait for GeneralFunctionPowSpec {
+    fn compute_hash(&self) -> HashBytes {
+        let mut hasher = blake3::Hasher::new();
+        hasher.update(uuid!("315d2847-a56d-49c8-8fef-5bcf710a7bc8").as_bytes()); // uuid for Pow
+        *hasher.finalize().as_bytes()
+    }
+
+    fn function(&self, tensors: &[Tensor], _device_dtype: &TorchDeviceDtype) -> Result<Tensor> {
+        Python::with_gil(|py| {
+            let pow = PY_UTILS
+                .pow
+                .call1(py, (tensors[0].clone(), tensors[1].clone()));
+            Ok(pow.context("evaluate pow")?.extract(py).unwrap())
+        })
+    }
+
+    fn get_shape_info(&self, shapes: &[Shape]) -> Result<GeneralFunctionShapeInfo> {
+        if shapes.len() != 2 {
+            bail!(GeneralFunctionShapeError::WrongNumShapes {
+                got: shapes.len(),
+                expected: 2
+            });
+        }
+        let shape = broadcast_shapes(&shapes)?;
+        Ok(GeneralFunctionShapeInfo {
+            shape: shape,
+            num_non_batchable_output_dims: 0,
+            input_batchability: vec![true, true],
+        })
+    }
+
+    fn name(&self) -> &'static str {
+        "pow"
+    }
+
+    fn is_official(&self) -> bool {
+        true
+    }
+
+    fn serialize(&self) -> Result<Option<String>> {
+        Ok(Some(self.name().to_owned()))
+    }
+}
+
+#[pyfunction]
+pub fn pow(base: CircuitRc, exponent: CircuitRc, name: Option<Name>) -> Result<GeneralFunction> {
+    GeneralFunction::try_new(
+        vec![base, exponent],
+        GeneralFunctionSpec::Pow(GeneralFunctionPowSpec {}),
+        name,
+    )
+}
+
+enum PyModuleLocator {
+    Path(String),
+    Module(String),
+}
+
+impl PyModuleLocator {
+    fn parse(s: String) -> Self {
+        if s.starts_with("/") && s.ends_with(".py") {
+            PyModuleLocator::Path(s)
+        } else {
+            PyModuleLocator::Module(s)
+        }
+    }
+
+    fn get_module<'py>(&self, py: Python<'py>) -> Result<&'py PyAny> {
+        match self {
+            PyModuleLocator::Path(path) => {
+                let rrfs_dir = py
+                    .import("interp.tools.rrfs")?
+                    .getattr("RRFS_DIR")?
+                    .extract::<String>()?;
+
+                // adapted from https://stackoverflow.com/questions/67631/how-can-i-import-a-module-dynamically-given-the-full-path
+
+                let importlib_util = py.import("importlib.util")?;
+                let spec = importlib_util
+                    .call_method1("spec_from_file_location", ("user_funcs", rrfs_dir + &path))?;
+                let function_spec_module =
+                    importlib_util.call_method1("module_from_spec", (spec,))?;
+                spec.getattr("loader")?
+                    .call_method1("exec_module", (function_spec_module,))?;
+                Ok(function_spec_module)
+            }
+            PyModuleLocator::Module(module) => Ok(py.import(&**module)?),
+        }
+    }
+}
+
+struct PyClassLocator {
+    module: PyModuleLocator,
+    name: String,
+}
+
+impl PyClassLocator {
+    fn parse(s: &String) -> Result<Self> {
+        return (|| -> Result<Self> {
+            let mut split = s.splitn(2, ":");
+            let file_path_or_module = convert_op_str_to_format_err(split.next(), s)?;
+            let spec_name = convert_op_str_to_format_err(split.next(), s)?;
+            Ok(PyClassLocator {
+                module: PyModuleLocator::parse(file_path_or_module),
+                name: spec_name.to_string(),
+            })
+        })().context(
+            "Function locator must either be of the form module.to.import:MyCustomSpec or /dir/sub_dir/.../file.py:MyCustomSpec (path relative to rrfs)"
+        );
+    }
+
+    fn get_class<'py>(&self, py: Python<'py>) -> Result<&'py PyAny> {
+        Ok(self.module.get_module(py)?.getattr(&*self.name)?)
+    }
+}
+
 #[derive(Clone)]
 pub struct PyWrap {
     ob: PyObject,
@@ -564,8 +696,8 @@ impl<'source> pyo3::FromPyObject<'source> for Box<PyWrap> {
         let path: Option<String> =
             Python::with_gil(|py| -> Result<_> { Ok(ob.getattr(py, "path")?.extract(py)?) })?;
 
-        if let Some(p) = &path && (!p.starts_with("/") || !p.contains(".py:")) {
-            return Err(anyhow!("Function path of the form /dir/sub_dir/.../file.py:MyCustomSpec. (relative to rrfs)").into());
+        if let Some(p) = &path {
+            PyClassLocator::parse(p)?;
         }
 
         Ok(Box::new(PyWrap {
@@ -662,6 +794,7 @@ pub enum GeneralFunctionSpec {
     Simple(GeneralFunctionSimpleSpec),
     Index(GeneralFunctionIndexSpec),
     SetDD(GeneralFunctionSetDDSpec),
+    Pow(GeneralFunctionPowSpec),
     Py(Box<PyWrap>), // boxed bc it was long pole for Circuit size
 }
 
@@ -672,6 +805,7 @@ impl GeneralFunctionSpec {
             Self::Simple(x) => x,
             Self::Index(x) => x,
             Self::SetDD(x) => x,
+            Self::Pow(x) => x,
             Self::Py(x) => x,
         }
     }
@@ -931,6 +1065,8 @@ impl GeneralFunction {
                 check_index_ints,
             }
             .into()
+        } else if spec_name.str() == "pow" {
+            GeneralFunctionPowSpec {}.into()
         } else if let Some(spec) = SPECS.get(&spec_name) {
             spec.clone()
         } else {
@@ -943,29 +1079,11 @@ impl GeneralFunction {
     #[staticmethod]
     #[args(nodes = "*", path, name = "None")]
     pub fn new_by_path(nodes: Vec<CircuitRc>, path: String, name: Option<Name>) -> Result<Self> {
-        let mut split = path.splitn(2, ":");
-        let file_path = convert_op_str_to_format_err(split.next(), &path)?;
-        let spec_name = convert_op_str_to_format_err(split.next(), &path)?;
+        let locator = PyClassLocator::parse(&path)?;
 
         let spec: GeneralFunctionSpec =
             Python::with_gil(|py| -> Result<GeneralFunctionSpec, pyo3::PyErr> {
-                let rrfs_dir = py
-                    .import("interp.tools.rrfs")?
-                    .getattr("RRFS_DIR")?
-                    .extract::<String>()?;
-
-                // adapted from https://stackoverflow.com/questions/67631/how-can-i-import-a-module-dynamically-given-the-full-path
-
-                let importlib_util = py.import("importlib.util")?;
-                let spec = importlib_util.call_method1(
-                    "spec_from_file_location",
-                    ("user_funcs", rrfs_dir + &file_path),
-                )?;
-                let function_spec_module =
-                    importlib_util.call_method1("module_from_spec", (spec,))?;
-                spec.getattr("loader")?
-                    .call_method1("exec_module", (function_spec_module,))?;
-                let spec_cls = function_spec_module.getattr(&*spec_name)?;
+                let spec_cls = locator.get_class(py)?;
                 spec_cls.call0()?.extract()
             })?;
 
