@@ -214,16 +214,18 @@ impl ModuleSpec {
             .context("batch shapes didn't match for module")?;
 
         // TODO: fix this being uncached!
-        let out = ReplaceExpander::new_noop().replace_expand_with_map(
-            self.circuit.clone(),
-            &ReplaceMapRc::new(
-                self.arg_specs
-                    .iter()
-                    .zip(nodes)
-                    .map(|(arg_spec, node)| (arg_spec.symbol.crc(), node.clone()))
-                    .collect(),
-            ),
-        )?;
+        let out = ReplaceExpander::new_noop()
+            .replace_expand_with_map(
+                self.circuit.clone(),
+                &ReplaceMapRc::new(
+                    self.arg_specs
+                        .iter()
+                        .zip(nodes)
+                        .map(|(arg_spec, node)| (arg_spec.symbol.crc(), node.clone()))
+                        .collect(),
+                ),
+            )
+            .context("replace expand failed from substitute")?;
 
         assert!(out.ndim() >= self.circuit.ndim());
         let out_batch_rank = out.ndim() - self.circuit.ndim();
@@ -377,36 +379,36 @@ impl ModuleSpec {
         let (expanded_shape, rep_hashes, clear_names_str) = self.expand_shape(&shapes)?;
         let node_mapping: HashMap<_, _> = rep_hashes.into_iter().zip(nodes).collect();
         // first update naming
-        let result = deep_map_op(expanded_shape.clone(), |x| {
-            // we only update naming if the given node has a bound free symbol
-            // (NOTE: these free symbols are now the *expanded* symbols from expand_shape)
-            if !x.is_symbol()
-                && x.info()
-                    .get_raw_free_symbols()
-                    .iter()
-                    .any(|x| node_mapping.contains_key(&x.info().hash))
-            {
-                if let Some(n) = x.info().name {
-                    if n.contains(&clear_names_str) {
-                        let name = x.get_autoname().unwrap_or(None);
-                        return Some(x.rename(name));
-                    } else if name_prefix.is_some() || name_suffix.is_some() {
-                        let name = Some(
-                            (name_prefix.as_deref().unwrap_or("").to_owned()
-                                + n.into()
-                                + name_suffix.as_deref().unwrap_or(""))
-                            .into(),
-                        );
-                        return Some(x.rename(name));
-                    }
+        let result = if name_prefix.is_some() || name_suffix.is_some() {
+            deep_map_op(expanded_shape.clone(), |x| {
+                // we only update naming if the given node has a bound free symbol
+                // (NOTE: these free symbols are now the *expanded* symbols from expand_shape)
+                if let Some(n) = x.info().name && !x.is_symbol()
+                    && x.info()
+                        .get_raw_free_symbols()
+                        .iter()
+                        .any(|x| node_mapping.contains_key(&x.info().hash))
+                {
+                    let name = Some(
+                        (name_prefix.as_deref().unwrap_or("").to_owned()
+                            + n.into()
+                            + name_suffix.as_deref().unwrap_or(""))
+                        .into(),
+                    );
+                    return Some(x.rename(name));
                 }
-            }
-            None
-        })
-        .unwrap_or(expanded_shape);
+                None
+            })
+            .unwrap_or(expanded_shape)
+        } else {
+            expanded_shape
+        };
         let result = deep_map(result, |x| {
             if let Some(r) = node_mapping.get(&x.info().hash) {
                 Ok((*r).clone())
+            } else if let Some(n) = x.info().name && n.contains(&clear_names_str) {
+                let name = x.get_autoname().unwrap_or(None);
+                Ok(x.rename(name))
             } else {
                 Ok(x)
             }
@@ -430,6 +432,22 @@ impl ModuleSpec {
         Ok(result)
     }
 
+    pub fn substitute_with_mod_name(
+        &self,
+        nodes: &Vec<CircuitRc>,
+        name_prefix: Option<String>,
+        name_suffix: Option<String>,
+        mod_name: Option<Name>,
+    ) -> Result<CircuitRc> {
+        let new_circ = self.substitute(nodes, name_prefix, name_suffix)?;
+        let out = if new_circ != self.circuit && self.circuit.num_children() > 0 {
+            new_circ.rename(mod_name)
+        } else {
+            new_circ
+        };
+        Ok(out)
+    }
+
     pub fn compute_non_children_hash(&self, hasher: &mut blake3::Hasher) {
         for arg_spec in &self.arg_specs {
             // this is fine because each item is fixed size and we delimit with node hashs (which are uu)
@@ -1245,7 +1263,7 @@ impl Module {
         self.substitute(self.info().name.map(|x| x.string() + "."), None)
     }
 
-    pub fn substitute(
+    pub fn substitute_no_self_rename(
         &self,
         name_prefix: Option<String>,
         name_suffix: Option<String>,
@@ -1253,7 +1271,16 @@ impl Module {
         self.spec
             .substitute(&self.nodes, name_prefix, name_suffix)
             .expect("constructor should ensure this works")
-            .rename(self.info().name)
+    }
+
+    pub fn substitute(
+        &self,
+        name_prefix: Option<String>,
+        name_suffix: Option<String>,
+    ) -> CircuitRc {
+        self.spec
+            .substitute_with_mod_name(&self.nodes, name_prefix, name_suffix, self.info().name)
+            .expect("constructor should ensure this works")
     }
 
     fn get_prefix(
@@ -1309,9 +1336,8 @@ impl AllModuleSubstituter {
             // but cached to O(n) per each module
             // and we could instead choose to make this O(n) but with worse caching.
             // Not clear if this would be slower or faster overall.
-            .substitute(&nodes, None, None)
+            .substitute_with_mod_name(&nodes, None, None, m.info().name)
             .expect("module constructor should ensure this works")
-            .rename(m.info().name)
         } else {
             circuit.map_children_unwrap_idxs(|i| new_children[i].clone())
         }
