@@ -9,7 +9,7 @@ use std::{
 
 use anyhow::{anyhow, bail, Context, Error, Result};
 use itertools::Itertools;
-use num_bigint::BigUint;
+use num_bigint::{BigInt, BigUint};
 use once_cell::unsync::Lazy;
 use pyo3::{once_cell::GILLazy, prelude::*};
 use rr_util::{
@@ -664,86 +664,7 @@ impl PrintOptions {
         let vs: &str = &variant_string;
         result.push_str(vs);
         result.push(' ');
-        result.push_str(&{
-            match &**circ {
-                Circuit::Scalar(scalar) => {
-                    format!("{:.}", scalar.value)
-                }
-                Circuit::Rearrange(rearrange) => rearrange.spec.to_maybe_fancy_einops_string(
-                    rearrange.node.info().named_axes.clone(),
-                    rearrange.info().named_axes.clone(),
-                ),
-                Circuit::Einsum(einsum) => einsum.get_spec().to_maybe_fancy_string(
-                    einsum.info().named_axes.clone(),
-                    einsum
-                        .children()
-                        .map(|c| c.info().named_axes.clone())
-                        .collect(),
-                ),
-                Circuit::Index(index) => {
-                    if self.bijection {
-                        index.index.repr_bijection(self.tensor_index_literal)?
-                    } else if self.tensor_index_literal {
-                        index.index.repr_bijection(self.tensor_index_literal)?
-                    } else {
-                        format!("{}", index.index)
-                    }
-                }
-                Circuit::Scatter(scatter) => {
-                    if self.bijection {
-                        scatter.index.repr_bijection(self.tensor_index_literal)?
-                    } else {
-                        format!("{}", scatter.index)
-                    }
-                }
-                Circuit::Concat(concat) => concat.axis.to_string(),
-                Circuit::GeneralFunction(gf) => {
-                    if self.bijection {
-                        gf.spec
-                            .serialize()
-                            .context("failed to get general function spec serialize in print")?
-                            .unwrap_or_else(|| format!("{} [NOT_SERIALIZABLE]", gf.spec.name()))
-                    } else {
-                        gf.spec.name().to_owned()
-                    }
-                }
-                Circuit::Symbol(sy) => {
-                    if sy.uuid.is_nil() {
-                        "".to_owned()
-                    } else {
-                        format!("{}", &sy.uuid)
-                    }
-                }
-                Circuit::Array(ac) => {
-                    if self.sync_tensors {
-                        ac.save_rrfs(true)?;
-                    }
-                    if self.bijection {
-                        if let Some(save_dir) = &self.tensor_save_dir {
-                            write_tensor_to_dir_tree(save_dir, ac.value.clone(), false)?;
-                        } else {
-                            ac.save_rrfs(false)?;
-                        }
-                        ac.tensor_hash_base16()[..24].to_owned()
-                    } else {
-                        "".to_owned()
-                    }
-                }
-                Circuit::Tag(at) => at.uuid.to_string(),
-                Circuit::StoredCumulantVar(scv) => {
-                    format!(
-                        "{}|{}",
-                        scv.cumulants
-                            .keys()
-                            .map(|k| k.to_string())
-                            .collect::<Vec<_>>()
-                            .join(", "),
-                        scv.uuid.to_string(),
-                    )
-                }
-                _ => "".to_owned(),
-            }
-        });
+        result.push_str(&self.repr_extra_info(circ)?);
 
         for _ in 0..2 {
             result = result
@@ -753,7 +674,86 @@ impl PrintOptions {
         }
         Ok(result)
     }
-
+    pub fn repr_extra_info(&self, circ: CircuitRc) -> Result<String> {
+        Ok(match &**circ {
+            Circuit::Scalar(scalar) => {
+                format!("{:.}", scalar.value)
+            }
+            Circuit::Rearrange(rearrange) => rearrange.spec.to_maybe_fancy_einops_string(
+                rearrange.node.info().named_axes.clone(),
+                rearrange.info().named_axes.clone(),
+            ),
+            Circuit::Einsum(einsum) => einsum.get_spec().to_maybe_fancy_string(
+                einsum.info().named_axes.clone(),
+                einsum
+                    .children()
+                    .map(|c| c.info().named_axes.clone())
+                    .collect(),
+            ),
+            Circuit::Index(index) => {
+                if self.bijection {
+                    index.index.repr_bijection(self.tensor_index_literal)?
+                } else if self.tensor_index_literal {
+                    index.index.repr_bijection(self.tensor_index_literal)?
+                } else {
+                    format!("{}", index.index)
+                }
+            }
+            Circuit::Scatter(scatter) => {
+                if self.bijection {
+                    scatter.index.repr_bijection(self.tensor_index_literal)?
+                } else {
+                    format!("{}", scatter.index)
+                }
+            }
+            Circuit::Concat(concat) => concat.axis.to_string(),
+            Circuit::GeneralFunction(gf) => {
+                if self.bijection {
+                    gf.spec
+                        .serialize()
+                        .context("failed to get general function spec serialize in print")?
+                        .unwrap_or_else(|| format!("{} [NOT_SERIALIZABLE]", gf.spec.name()))
+                } else {
+                    gf.spec.name().to_owned()
+                }
+            }
+            Circuit::Symbol(sy) => {
+                if sy.uuid.is_nil() {
+                    "".to_owned()
+                } else {
+                    format!("{}", &sy.uuid)
+                }
+            }
+            Circuit::Array(ac) => {
+                if self.sync_tensors {
+                    ac.save_rrfs(true)?;
+                }
+                if self.bijection {
+                    if let Some(save_dir) = &self.tensor_save_dir {
+                        write_tensor_to_dir_tree(save_dir, ac.value.clone(), false)?;
+                    } else {
+                        ac.save_rrfs(false)?;
+                    }
+                    ac.tensor_hash_base16()[..24].to_owned()
+                } else {
+                    "".to_owned()
+                }
+            }
+            Circuit::Tag(at) => at.uuid.to_string(),
+            Circuit::StoredCumulantVar(scv) => {
+                format!(
+                    "{}|{}",
+                    scv.cumulants
+                        .keys()
+                        .map(|k| k.to_string())
+                        .collect::<Vec<_>>()
+                        .join(", "),
+                    scv.uuid.to_string(),
+                )
+            }
+            _ => "".to_owned(),
+        })
+    }
     #[pyo3(name = "repr")]
     #[args(circuits = "*")]
     pub fn repr_circuits(&self, circuits: Vec<CircuitRc>) -> Result<String> {
@@ -1179,6 +1179,12 @@ pub fn debug_repr(circ: CircuitRc) -> Result<String> {
     ))
 }
 
+#[pyfunction]
+#[pyo3(name = "oom_fmt")]
+pub fn oom_fmt_py(t: BigUint) -> String {
+    oom_fmt(t)
+}
+
 pub fn oom_fmt<T: Into<BigUint>>(num: T) -> String {
     let mut num: BigUint = num.into();
     let k = BigUint::from(1000usize);
