@@ -4,6 +4,7 @@ use anyhow::{bail, Result};
 use macro_rules_attribute::apply;
 use pyo3::prelude::*;
 use rr_util::{
+    name::Name,
     py_types::Tensor,
     sv,
     tensor_util::{Shape, TorchDeviceDtype},
@@ -34,19 +35,18 @@ pub struct Tag {
     pub node: CircuitRc,
     #[pyo3(get)]
     pub uuid: Uuid,
-    name: Option<String>,
     info: CachedCircuitInfo,
 }
 
 impl Tag {
     #[apply(new_rc)]
-    pub fn new(node: CircuitRc, uuid: Uuid, name: Option<String>) -> (Self) {
-        let out = Self {
+    pub fn new(node: CircuitRc, uuid: Uuid, name: Option<Name>) -> (Self) {
+        let mut out = Self {
             node,
             uuid,
-            name,
             info: Default::default(),
         };
+        out.info.name = name;
         out.initial_init_info().unwrap()
     }
 }
@@ -54,8 +54,9 @@ impl Tag {
 impl CircuitNodeAutoName for Tag {
     const PRIORITY: OperatorPriority = OperatorPriority::PostFix {};
 
-    fn auto_name(&self) -> Option<String> {
-        child_name_with_maybe_paren(&Self::PRIORITY, self.node.clone()).map(|n| n + " tag")
+    fn auto_name(&self) -> Option<Name> {
+        child_name_with_maybe_paren(&Self::PRIORITY, self.node.clone())
+            .map(|n| (n.string() + " tag").into())
     }
 }
 
@@ -92,7 +93,7 @@ impl CircuitNode for Tag {
         Ok(Self::new(
             f(0, self.node.clone())?,
             self.uuid,
-            self.name.clone(),
+            self.info().name,
         ))
     }
 
@@ -108,7 +109,7 @@ impl Tag {
     fn py_new(
         node: CircuitRc,
         uuid: Uuid,
-        name: Option<String>,
+        name: Option<Name>,
     ) -> PyResult<PyClassInitializer<Tag>> {
         let out = Self::new(node, uuid, name);
         Ok(out.into_init())
@@ -116,7 +117,7 @@ impl Tag {
 
     /// Creates a new Tag with a random UUID.
     #[staticmethod]
-    pub fn new_with_random_uuid(node: CircuitRc, name: Option<String>) -> Self {
+    pub fn new_with_random_uuid(node: CircuitRc, name: Option<Name>) -> Self {
         Self::new(node, Uuid::new_v4(), name)
     }
 }
@@ -128,7 +129,6 @@ pub struct DiscreteVar {
     pub values: CircuitRc,
     #[pyo3(get)]
     pub probs_and_group: CircuitRc,
-    name: Option<String>,
     info: CachedCircuitInfo,
 }
 
@@ -137,7 +137,7 @@ impl DiscreteVar {
     pub fn try_new(
         values: CircuitRc,
         probs_and_group: CircuitRc,
-        name: Option<String>,
+        name: Option<Name>,
     ) -> Result<Self> {
         if probs_and_group.info().rank() != 1 {
             bail!(ConstructError::DiscreteVarProbsMustBe1d {
@@ -153,19 +153,19 @@ impl DiscreteVar {
                 probs: probs_and_group.info().shape[0],
             });
         }
-        let out = Self {
+        let mut out = Self {
             values,
             probs_and_group,
-            name,
             info: Default::default(),
         };
+        out.info.name = name;
         out.initial_init_info()
     }
 
     pub fn new_for_py(
         values: CircuitRc,
         probs_and_group: Option<CircuitRc>,
-        name: Option<String>,
+        name: Option<Name>,
     ) -> Result<Self> {
         if values.info().rank() < 1 {
             bail!(ConstructError::DiscreteVarNoSamplesDim {});
@@ -179,9 +179,10 @@ impl DiscreteVar {
 impl CircuitNodeAutoName for DiscreteVar {
     const PRIORITY: OperatorPriority = OperatorPriority::PostFix {};
 
-    fn auto_name(&self) -> Option<String> {
+    fn auto_name(&self) -> Option<Name> {
         // Ignores probs and groups because we almost never care about the probs and groups name
-        child_name_with_maybe_paren(&Self::PRIORITY, self.values.clone()).map(|n| n + " var")
+        child_name_with_maybe_paren(&Self::PRIORITY, self.values.clone())
+            .map(|n| (n.string() + " var").into())
     }
 }
 
@@ -225,7 +226,7 @@ impl CircuitNode for DiscreteVar {
         Self::try_new(
             f(0, self.values.clone())?,
             f(1, self.probs_and_group.clone())?,
-            self.name.clone(),
+            self.info().name,
         )
     }
 
@@ -247,17 +248,17 @@ impl DiscreteVar {
     fn py_new(
         values: CircuitRc,
         probs_and_group: Option<CircuitRc>,
-        name: Option<String>,
+        name: Option<Name>,
     ) -> Result<PyClassInitializer<DiscreteVar>> {
         let out = Self::new_for_py(values, probs_and_group, name)?;
         Ok(out.into_init())
     }
     #[staticmethod]
-    pub fn new_uniform(values: CircuitRc, name: Option<String>) -> Result<Self> {
+    pub fn new_uniform(values: CircuitRc, name: Option<Name>) -> Result<Self> {
         Self::new_for_py(values, None, name)
     }
     #[staticmethod]
-    pub fn uniform_probs_and_group(size: usize, name: Option<String>) -> Tag {
+    pub fn uniform_probs_and_group(size: usize, name: Option<Name>) -> Tag {
         // TODO: naming
         Tag::new_with_random_uuid(Scalar::nrc(1.0 / (size as f64), sv![size], None), name)
     }
@@ -270,7 +271,6 @@ pub struct StoredCumulantVar {
     pub cumulants: BTreeMap<usize, CircuitRc>, // using btreemap so it always iterates in order
     #[pyo3(get)]
     pub uuid: Uuid,
-    name: Option<String>,
     info: CachedCircuitInfo,
 }
 
@@ -279,7 +279,7 @@ impl StoredCumulantVar {
     pub fn try_new(
         cumulants: BTreeMap<usize, CircuitRc>,
         uuid: Uuid,
-        name: Option<String>,
+        name: Option<Name>,
     ) -> Result<Self> {
         if !cumulants.contains_key(&1) || !cumulants.contains_key(&2) {
             bail!(ConstructError::StoredCumulantVarNeedsMeanVariance {});
@@ -303,12 +303,12 @@ impl StoredCumulantVar {
                 },);
             }
         }
-        let out = Self {
+        let mut out = Self {
             cumulants,
             uuid,
-            name,
             info: Default::default(),
         };
+        out.info.name = name;
         out.initial_init_info()
     }
 
@@ -317,7 +317,7 @@ impl StoredCumulantVar {
         variance: CircuitRc,
         higher_cumulants: BTreeMap<usize, CircuitRc>,
         uuid: Option<Uuid>,
-        name: Option<String>,
+        name: Option<Name>,
     ) -> Result<Self> {
         let mut higher_cumulants = higher_cumulants;
         higher_cumulants.insert(1, mean);
@@ -331,21 +331,25 @@ impl StoredCumulantVar {
 }
 
 impl CircuitNodeAutoName for StoredCumulantVar {
-    fn auto_name(&self) -> Option<String> {
-        if self.cumulants.values().any(|x| x.name().is_none()) {
+    fn auto_name(&self) -> Option<Name> {
+        if self.cumulants.values().any(|x| x.info().name.is_none()) {
             None
         } else {
             Some(
-                "StoredCumulantVar(".to_owned()
+                ("StoredCumulantVar(".to_owned()
                     + &self
                         .cumulants
                         .iter()
                         .map(|(r, x)| {
-                            format!("{r}: {}", Self::shorten_child_name(x.name().unwrap()))
+                            format!(
+                                "{r}: {}",
+                                Self::shorten_child_name(x.info().name.unwrap().str())
+                            )
                         })
                         .collect::<Vec<String>>()
                         .join(", ")
-                    + ")",
+                    + ")")
+                    .into(),
             )
         }
     }
@@ -405,7 +409,7 @@ impl CircuitNode for StoredCumulantVar {
                 .map(|(i, (k, v))| f(i, v.clone()).map(|z| (*k, z)))
                 .collect::<Result<BTreeMap<_, _>, _>>()?,
             self.uuid,
-            self.name.clone(),
+            self.info().name,
         )
     }
 
@@ -427,7 +431,7 @@ impl StoredCumulantVar {
     fn py_new(
         cumulants: BTreeMap<usize, CircuitRc>,
         uuid: Option<Uuid>,
-        name: Option<String>,
+        name: Option<Name>,
     ) -> PyResult<PyClassInitializer<StoredCumulantVar>> {
         let uuid = uuid.unwrap_or_else(|| Uuid::new_v4());
         let out = Self::try_new(cumulants, uuid, name)?;
@@ -442,7 +446,7 @@ impl StoredCumulantVar {
         variance: CircuitRc,
         higher_cumulants: BTreeMap<usize, CircuitRc>,
         uuid: Option<Uuid>,
-        name: Option<String>,
+        name: Option<Name>,
     ) -> Result<Self> {
         Self::new_mv(mean, variance, higher_cumulants, uuid, name)
     }
