@@ -2,10 +2,12 @@ use std::{
     collections::BTreeSet,
     fmt::{Display, Write},
     iter::{once, zip},
+    sync::Arc,
 };
 
 use anyhow::Result;
 use circuit_base::{
+    circuit_utils::toposort_circuit,
     clicolor,
     print::{color, last_child_arrows, CliColor, PrintOptions},
     CircuitNode, CircuitNodeUnion, CircuitRc,
@@ -17,7 +19,7 @@ use rr_util::{
     python_println,
     util::{indent, indent_butfirst},
 };
-use rustc_hash::FxHashSet as HashSet;
+use rustc_hash::{FxHashMap as HashMap, FxHashSet as HashSet};
 
 use crate::{
     iterative_matcher::{per_child, ChainItem, IterativeMatcher},
@@ -270,6 +272,80 @@ impl Matcher {
     }
 }
 
+// pub struct MatcherTreeifyItem {
+//     circuit: CircuitRc,
+//     matchers: BTreeSet<IterativeMatcherRc>,
+//     children: Vec<Arc<MatcherTreeifyItem>>,
+// }
+// ///
+// pub fn matcher_treeify_by_later_behavior(
+//     circuit: CircuitRc,
+//     matcher: IterativeMatcherRc,
+// ) -> Result<MatcherTreeifyItem> {
+//     type Key = (CircuitRc, IterativeMatcherRc);
+//     let mut later_matched: HashMap<Key, BTreeSet<CircuitRc>> = Default::default();
+//     fn recurse(
+//         circuit: CircuitRc,
+//         matcher: IterativeMatcherRc,
+//         later_matched: &mut HashMap<Key, BTreeSet<CircuitRc>>,
+//         upstream_stack: &mut Vec<Key>,
+//     ) -> Result<()> {
+//         let key = (circuit.clone(), matcher.clone());
+//         if later_matched.contains_key(&key) {
+//             return Ok(());
+//         }
+//         later_matched.insert(key.clone(), Default::default());
+//         let matched: IterateMatchResults = matcher.match_iterate(circuit.clone())?;
+//         if matched.found {
+//             for us in upstream_stack.iter() {
+//                 later_matched.get_mut(us).unwrap().insert(circuit.clone());
+//             }
+//         }
+//         let child_matchers =
+//             per_child(matched.updated, matcher.clone(), circuit.children().count());
+
+//         upstream_stack.push(key);
+//         for (i, (child, child_matcher)) in zip(circuit.children(), child_matchers).enumerate() {
+//             if let Some(child_matcher) = child_matcher {
+//                 let child_key = (child.clone(), child_matcher.clone());
+//                 recurse(child, child_matcher, later_matched, upstream_stack)?;
+//             }
+//         }
+//         upstream_stack.pop();
+
+//         Ok(())
+//     }
+//     recurse(
+//         circuit.clone(),
+//         matcher.clone(),
+//         &mut later_matched,
+//         &mut vec![],
+//     )?;
+//     let topo = toposort_circuit(circuit);
+//     let mut set_to_key: HashMap<BTreeSet<CircuitRc>, Vec<Key>> = Default::default();
+//     for (k, v) in &later_matched {
+//         set_to_key
+//             .entry(v.clone())
+//             .or_insert(Default::default())
+//             .push(k.clone());
+//     }
+//     /// for
+//     fn recurse_2(
+//         key: Key,
+//         matcher_set: &BTreeSet<CircuitRc>,
+//         later_matched: &HashMap<Key, BTreeSet<CircuitRc>>,
+//         set_to_key: &HashMap<BTreeSet<CircuitRc>, Vec<Key>>,
+//     ) -> MatcherTreeifyItem {
+//         let setty = later_matched[&key];
+//     }
+//     Ok(recurse_2(
+//         (circuit.clone(), matcher.clone()),
+//         &later_matched[&(circuit.clone(), matcher.clone())],
+//         &later_matched,
+//         &set_to_key,
+//     ))
+// }
+
 #[pyfunction(print_options = "Default::default()")]
 pub fn repr_matcher_debug(
     circuit: CircuitRc,
