@@ -465,7 +465,7 @@ pub struct ModulePusher {
             HashBytes,
             HashBytes,
         ),
-        CircOrGet,
+        (CircOrGet, (bool, HashSet<CircuitRc>)),
     >,
     replace_expander: MapReplaceExpander,
 }
@@ -644,6 +644,7 @@ impl ModulePusher {
             &Default::default(),
             &Default::default(),
         )
+        .map(|x| x.0)
     }
 
     // we could do a generic function on the bools, but I assume this has better compile time
@@ -667,7 +668,7 @@ impl ModulePusher {
         mode: PushDownMode,
         items: &NestedModuleItems,
         extra_replacements: &ReplaceMapRc,
-    ) -> Result<CircOrGet> {
+    ) -> Result<(CircOrGet, (bool, HashSet<CircuitRc>))> {
         let push_or_get = mode == PushDownMode::PushOverride || mode == PushDownMode::Get;
         if push_or_get {
             assert_eq!(extra_replacements.len(), 0);
@@ -700,6 +701,8 @@ impl ModulePusher {
         let found_override = found_get && mode == PushDownMode::PushOverride;
         let all_done = all_finished || found_override || no_child_will_get;
 
+        let extra_default = (true, Default::default());
+
         if all_done {
             let out = match (found_override, mode) {
                 (_, PushDownMode::Get) => CircOrGet::Get(out_set),
@@ -708,7 +711,7 @@ impl ModulePusher {
                 }
                 (false, PushDownMode::PushOverride) => CircOrGet::Circ(circuit),
             };
-            return Ok(out);
+            return Ok((out, extra_default));
         }
 
         if let Some(arg) = circuit
@@ -722,10 +725,16 @@ impl ModulePusher {
                 })
             }
 
-            return Ok(CircOrGet::Circ(arg.clone()));
+            return Ok((
+                CircOrGet::Circ(arg.clone()),
+                (true, [arg.clone()].into_iter().collect()),
+            ));
         }
         if let Some(out) = extra_replacements.get(&circuit) {
-            return Ok(CircOrGet::Circ(out.clone()));
+            return Ok((
+                CircOrGet::Circ(out.clone()),
+                (true, [out.clone()].into_iter().collect()),
+            ));
         }
 
         let (new_skip_module, found_skip_module) = skip_module
@@ -736,18 +745,18 @@ impl ModulePusher {
         let mut rec = |self_: &mut Self, c, get, skip, items, extra_replacements: &_| match self_
             .push_down_modules_rec(c, get, skip, mode, items, extra_replacements)?
         {
-            CircOrGet::Circ(c) => {
+            (CircOrGet::Circ(c), b) => {
                 assert!(!is_get);
-                Ok(Some(c))
+                Ok((Some(c), b))
             }
-            CircOrGet::Get(new_set) => {
+            (CircOrGet::Get(new_set), b) => {
                 assert!(is_get);
                 out_set.extend(new_set);
-                Ok(None)
+                Ok((None, b))
             }
         };
 
-        let out = if let Some(m) = circuit
+        let (out, prior_inner_circuits, circuit_is_from_term) = if let Some(m) = circuit
             .as_module()
             .and_then(|x| (!found_skip_module).then_some(x))
         {
@@ -762,7 +771,9 @@ impl ModulePusher {
                 .clone()
                 .into_iter()
                 .zip(node_get_skip.clone())
-                .map(|(c, (get, skip))| rec(self_, c, get, skip, items, extra_replacements))
+                .map(|(c, (get, skip))| {
+                    rec(self_, c, get, skip, items, extra_replacements).map(|x| x.0)
+                })
                 .collect::<Result<Vec<_>>>()?;
 
             let (new_nodes_flat, new_nodes_base): (Vec<_>, _) = if push_or_get {
@@ -770,6 +781,7 @@ impl ModulePusher {
                 let new_nodes_base = m.nodes.clone();
 
                 // we fully substitute in the 'get' case
+                // Argueably, this should push down for naming
 
                 (
                     m.nodes
@@ -806,7 +818,7 @@ impl ModulePusher {
                             mode,
                             &Default::default(), // no items (only difference from recursive call above)
                             extra_replacements,
-                        )?, CircOrGet::Circ);
+                        )?.0, CircOrGet::Circ);
                         assert_eq!(out.ndim(), ndim);
                         Ok(out)
                     })
@@ -820,7 +832,7 @@ impl ModulePusher {
                 )
             };
 
-            let out = rec(
+            let (out, (termed_prior_iter, mut prior_inner_circuits)) = rec(
                 self_,
                 m.spec.circuit.clone(),
                 spec_circuit_get,
@@ -829,7 +841,7 @@ impl ModulePusher {
                 extra_replacements,
             )?;
             if mode == PushDownMode::PushOverride {
-                Some(
+                let out = Some(
                     m.map_children_unwrap_idxs(|i| {
                         if i == 0 {
                             out.clone().unwrap()
@@ -838,9 +850,11 @@ impl ModulePusher {
                         }
                     })
                     .rc(),
-                )
+                );
+                (out, Default::default(), false)
             } else {
-                out
+                prior_inner_circuits.insert(m.spec.circuit.clone());
+                (out, prior_inner_circuits, termed_prior_iter)
             }
         } else {
             if let Some(m) = circuit.as_module() {
@@ -861,11 +875,11 @@ impl ModulePusher {
                 .zip(new_get.clone())
                 .zip(new_skip_module.clone())
                 .zip(extra_replacements.per_child(&circuit))
-                .map(|(((c, get), skip), rep)| rec(self_, c, get, skip, items, &rep))
+                .map(|(((c, get), skip), rep)| rec(self_, c, get, skip, items, &rep).map(|x| x.0))
                 .collect::<Result<Vec<_>>>()?;
 
             if is_get {
-                return Ok(CircOrGet::Get(out_set));
+                return Ok((CircOrGet::Get(out_set), extra_default));
             }
 
             let out = expand_node(
@@ -880,7 +894,7 @@ impl ModulePusher {
                         items,
                         &extra_replacements.extend_into(rep),
                     )
-                    .map(|x| x.unwrap())
+                    .map(|x| x.0.unwrap())
                 },
             )
             // maybe we're supposed to just panic on rearrange rank errors?
@@ -888,16 +902,23 @@ impl ModulePusher {
                 "expand fail in push down modules, should be rearrange ",
                 "overflow error (otherwise internal error)"
             ))?;
-            Some(out)
+            (Some(out), Default::default(), false)
         };
 
         if is_get {
-            return Ok(CircOrGet::Get(out_set));
+            return Ok((CircOrGet::Get(out_set), extra_default));
         }
         let out = out.unwrap();
 
         // rename if we didn't create new
-        let out = if out != circuit {
+        //
+        // we also special case module 'stacks' with no intermediate circuits:
+        // this is what prior_spec_circuits and circuit_is_from_term are for.
+        let out = if out != circuit
+            && !prior_inner_circuits.contains(&out)
+            // if at top of module stack, ignore from term
+            && (!circuit_is_from_term || items.modules.is_empty())
+        {
             let new_name = self_
                 .namer
                 .call(
@@ -912,7 +933,10 @@ impl ModulePusher {
             out
         };
 
-        Ok(CircOrGet::Circ(out))
+        Ok((
+            CircOrGet::Circ(out),
+            (circuit_is_from_term, prior_inner_circuits),
+        ))
     }
 
     fn finalize(
