@@ -10,6 +10,7 @@ use macro_rules_attribute::apply;
 use pyo3::{exceptions::PyValueError, prelude::*};
 use rr_util::{
     compact_data::U8Set,
+    name::Name,
     python_error_exception,
     rearrange_spec::{check_permutation, PermError},
     union_find::UnionFind,
@@ -277,12 +278,12 @@ macro_rules! nest_spec_for {
 }
 
 nest_spec_for!(Einsum {
-    name: Option<String>,
+    name: Option<rr_util::name::Name>,
     out_axes_perm: Option<rr_util::util::EinsumAxes>,
     shrink_out_axes: bool,
 });
 nest_spec_for!(Add {
-    name: Option<String>,
+    name: Option<rr_util::name::Name>,
 });
 
 impl Default for NestEinsumsInfo {
@@ -321,7 +322,7 @@ impl NestInfo for NestAddsInfo {
 
 impl NestEinsumsInfo {
     pub fn new(
-        name: Option<String>,
+        name: Option<Name>,
         out_axes_perm: Option<EinsumAxes>,
         shrink_out_axes: bool,
     ) -> Result<Self> {
@@ -336,7 +337,7 @@ impl NestEinsumsInfo {
     }
 }
 impl NestAddsInfo {
-    pub fn new(name: Option<String>) -> Result<Self> {
+    pub fn new(name: Option<Name>) -> Result<Self> {
         Ok(Self { name })
     }
 }
@@ -362,7 +363,7 @@ pub enum NestIdxsItem<I: NestFullInfo> {
 
 #[derive(Debug, Clone)]
 pub struct NestEinsumsFullInfo {
-    pub name: Option<String>,
+    pub name: Option<Name>,
     pub out_axes: Option<EinsumAxes>,
     pub out_axes_perm: Option<EinsumAxes>,
     pub shrink_out_axes: bool,
@@ -371,7 +372,7 @@ type NestAddsFullInfo = NestAddsInfo;
 
 impl NestEinsumsFullInfo {
     fn new(
-        name: Option<String>,
+        name: Option<Name>,
         out_axes: Option<EinsumAxes>,
         out_axes_perm: Option<EinsumAxes>,
         shrink_out_axes: bool,
@@ -399,7 +400,7 @@ impl NestFullInfo for NestEinsumsFullInfo {
 
     fn from_circ(circ: CircuitRc) -> Self {
         Self {
-            name: circ.name_cloned(),
+            name: circ.info().name,
             out_axes: circ.as_einsum().map(|x| x.out_axes.clone()),
             ..Default::default()
         }
@@ -445,7 +446,7 @@ impl NestFullInfo for NestEinsumsFullInfo {
             })
             .collect();
 
-        let out_circ = Einsum::nrc(new_args, einsum.out_axes.clone(), einsum.name_cloned());
+        let out_circ = Einsum::nrc(new_args, einsum.out_axes.clone(), einsum.info().name);
 
         finish_with_extra_and_circ(&rec_on_children, &extra, out_circ)
     }
@@ -454,7 +455,7 @@ impl NestFullInfo for NestEinsumsFullInfo {
         flat: &Self::CircuitType,
         specs: Vec<NestIdxsInfo<Self>>,
     ) -> Result<Self::CircuitType> {
-        nest_flat_einsum_strict_rec(&flat.args, flat.out_axes.clone(), specs, flat.name_cloned())
+        nest_flat_einsum_strict_rec(&flat.args, flat.out_axes.clone(), specs, flat.info().name)
     }
 }
 impl NestFullInfo for NestAddsFullInfo {
@@ -463,7 +464,7 @@ impl NestFullInfo for NestAddsFullInfo {
 
     fn from_circ(circ: CircuitRc) -> Self {
         Self {
-            name: circ.name_cloned(),
+            name: circ.info().name,
         }
     }
 
@@ -483,7 +484,7 @@ impl NestFullInfo for NestAddsFullInfo {
                 }
             })
             .collect();
-        let out_circ = Add::nrc(new_operands, add.name_cloned());
+        let out_circ = Add::nrc(new_operands, add.info().name);
 
         finish_with_extra_and_circ(&rec_on_children, &vec![(); rec_on_children.len()], out_circ)
     }
@@ -495,7 +496,7 @@ impl NestFullInfo for NestAddsFullInfo {
         Ok(nest_flat_add_strict_rec(
             &flat.nodes,
             specs,
-            flat.name_cloned(),
+            flat.info().name,
         ))
     }
 }
@@ -1151,7 +1152,7 @@ pub fn nest_flat_einsum_strict_rec(
     flat_einsum: &EinsumArgs,
     out_axes: EinsumAxes,
     specs: Vec<NestIdxsInfo<NestEinsumsFullInfo>>,
-    name: Option<String>,
+    name: Option<Name>,
 ) -> Result<Einsum> {
     // TODO: avoid quadratic running time...
     let all_ints: Vec<U8Set> = specs
@@ -1239,7 +1240,7 @@ pub fn nest_flat_einsum_strict_rec(
 pub fn nest_flat_add_strict_rec(
     flat_nodes: &[CircuitRc],
     specs: Vec<NestIdxsInfo<NestAddsFullInfo>>,
-    name: Option<String>,
+    name: Option<Name>,
 ) -> Add {
     let nodes = specs
         .into_iter()
