@@ -11,6 +11,7 @@ use circuit_rewrites::{
 };
 use mimalloc::MiMalloc;
 use rr_util::{
+    name::{Name, NameInterner},
     opt_einsum::{get_disconnected_sugraphs, get_int_to_tensor_appearance, EinsumSpec},
     rrfs::get_rrfs_dir,
     sv,
@@ -64,9 +65,17 @@ pub fn main() {
     opsettings.verbose = 2;
     opsettings.log_simplifications = true;
     opsettings.keep_all_names = true;
-    for _ in 0..4 {
+    for _ in 0..3 {
         let mut ctx = OptimizationContext::new_settings(opsettings);
-        timed!(optimize_circuit(circuits_allnamed[0].clone(), &mut ctx).unwrap());
+        for circ in &circuits_allnamed {
+            println!("{:?}", circ.info().name);
+            timed!(optimize_circuit(circ.clone(), &mut ctx).unwrap());
+        }
         println!("{}", ctx.stringify_logs());
     }
+    println!(
+        "interned num {} len {}",
+        NameInterner::num_interned_strings(),
+        NameInterner::total_interned_string_len()
+    );
 }
