@@ -16,14 +16,10 @@ fn benchmarking_circs() {
         let circuits: Vec<_> = fs::read_dir(format!("{}/ryan/compiler_benches/", get_rrfs_dir()))
             .unwrap()
             .filter_map(|d| {
-                Parser::new(
-                    Default::default(),
-                    Default::default(),
-                    true,
-                    Default::default(),
-                    true,
-                )
-                .unwrap()
+                Parser {
+                    tensors_as_random: true,
+                    ..Default::default()
+                }
                 .parse_circuit(&fs::read_to_string(d.unwrap().path()).unwrap(), &mut None)
                 .ok()
             })
